@@ -1,6 +1,5 @@
 package Posy;
 use strict;
-use warnings;
 
 =head1 NAME
 
@@ -8,11 +7,11 @@ Posy - a website generator inspired by blosxom
 
 =head1 VERSION
 
-This describes version B<0.01> of Posy.
+This describes version B<0.02> of Posy.
 
 =cut
 
-our $VERSION = '0.01';
+our $VERSION = '0.02';
 
 =head1 SYNOPSIS
 
@@ -32,10 +31,6 @@ formats can be dealt with if one writes a plugin to deal with them.
 
 =cut
 
-use File::Spec;
-use File::stat;
-use FileHandle;
-
 =head1 CLASS METHODS
 
 =head2 import
@@ -76,13 +71,11 @@ sub import {
 
     File::Spec
     File::stat
-    FileHandle
 
     File::Find
     Data::Dumper
     Storable
-    Carp
-    CGI
+    CGI::Minimal
 
     Test::More
 
